'use strict';

var react = require('react');
var shim = require('use-sync-external-store/shim');

let run = (fn) => {
    fn();
};
const __DEV__ = process.env.NODE_ENV !== 'production';
const create = (initStore) => {
    if (__DEV__ && typeof initStore !== 'function') {
        throw new Error('Expected a function');
    }
    let store;
    const listeners = new Set();
    function getSetStore(s) {
        if (typeof s === 'undefined')
            return store;
        const partial = typeof s === 'function' ? s(store) : s;
        store = Object.assign(Object.assign({}, store), partial);
        run(() => listeners.forEach((listener) => listener(partial)));
    }
    store = initStore(getSetStore);
    return function useStore() {
        const proxy = react.useRef({});
        const handler = react.useRef({});
        const hasState = react.useRef(false);
        const hasUpdate = react.useRef(false);
        const [, setState] = react.useState(false);
        react.useMemo(() => {
            handler.current = {
                get(target, key) {
                    const val = store[key];
                    if (typeof val !== 'function') {
                        hasState.current = true;
                        target[key] = val;
                        return target[key];
                    }
                    target[key] = new Proxy(val, {
                        get: (fn, fnKey) => {
                            if (fnKey === 'loading' && !('loading' in fn)) {
                                fn.loading = false;
                            }
                            return fn[fnKey];
                        },
                        apply: (fn, _this, args) => {
                            const res = fn(...args);
                            if (!('loading' in fn) ||
                                !res ||
                                typeof res.then !== 'function') {
                                target[key] = fn;
                                return res;
                            }
                            const setLoading = (loading) => {
                                target[key].loading = loading;
                                setState((s) => !s);
                            };
                            target[key] = ((...newArgs) => {
                                const newRes = fn(...newArgs);
                                setLoading(true);
                                return newRes.finally(() => setLoading(false));
                            });
                            setLoading(true);
                            return res.finally(() => setLoading(false));
                        },
                    });
                    return target[key];
                },
                set(target, key, val) {
                    if (key in target && val !== target[key]) {
                        hasUpdate.current = true;
                        target[key] = val;
                    }
                    return true;
                },
            };
            proxy.current = new Proxy({}, handler.current);
        }, []);
        react.useEffect(() => {
            handler.current.get = (target, key) => target[key];
        }, []);
        const subscribe = react.useCallback((update) => {
            if (!hasState.current)
                return () => undefined;
            const listener = (partial) => {
                Object.assign(proxy.current, partial);
                if (hasUpdate.current) {
                    hasUpdate.current = false;
                    update();
                }
            };
            listeners.add(listener);
            return () => listeners.delete(listener);
        }, []);
        const getSnapshot = react.useCallback(() => store, []);
        shim.useSyncExternalStore(subscribe, getSnapshot);
        return proxy.current;
    };
};
create.config = ({ batch }) => {
    run = batch;
};

module.exports = create;
